#include "luminosity.hh"
#include "QCD_EW.hh"
#include "LHAPDF_interface.hh"
#include "math/integration.hh"


using namespace std;



namespace lumi {

  // Higgs part
  double lumi_gg(double x1, double x2, double mu) {
    return PDF::g(x1,mu) * PDF::g(x2,mu);
  }





  // Drell-Yan part
  //
  const double V_ud = EW::vud;//0.975;
  const double V_us = EW::vus;//0.222;
  const double V_cd = EW::vcd;//0.222;
  const double V_cs = EW::vcs;//0.974;
  const double V_ud_sq = V_ud*V_ud;     double V_us_sq = V_us*V_us;
  const double V_cd_sq = V_cd*V_cd;     double V_cs_sq = V_cs*V_cs;
  const double V_ub_sq = 1. - V_ud_sq - V_us_sq;
  const double V_cb_sq = 1. - V_cd_sq - V_cs_sq;
  const double V_td_sq = 1. - V_ud_sq - V_cd_sq;
  const double V_ts_sq = 1. - V_us_sq - V_cs_sq;
  const double V_tb_sq = 1. - V_ub_sq - V_cb_sq;
  //
  double N_Zgamma(double Q, double alpha){
    double Qsq = Q*Q;  double mZsq = MASS::mass_Z*MASS::mass_Z;
    return (1.-4.*EW::s2W)/8./EW::s2W/EW::c2W * Qsq*(Qsq-mZsq)/((Qsq-mZsq)*(Qsq-mZsq) + EW::Gamma_Z*EW::Gamma_Z * mZsq);
  }
  double N_Z(double Q, double alpha){
    double Qsq = Q*Q;  double mZsq = MASS::mass_Z*MASS::mass_Z;
    return 3./16./EW::s2W/EW::c2W/alpha * EW::Gamma_Z * EW::Br_l_Z/MASS::mass_Z
      * Qsq*Qsq/((Qsq-mZsq)*(Qsq-mZsq) + EW::Gamma_Z*EW::Gamma_Z * mZsq);
  }
  const double a_u_Z = -1., a_d_Z = 1.;
  const double v_u_Z = 1.-8./3.*EW::s2W, v_d_Z = -1.+4./3.*EW::s2W;
  double VsqAsq_u(double M, double alpha){
    return EW::e_u*EW::e_u + N_Zgamma(M,alpha)*EW::e_u*v_u_Z + N_Z(M,alpha)*(v_u_Z*v_u_Z + a_u_Z*a_u_Z);
  }
  double VsqAsq_d(double M, double alpha){
    return EW::e_d*EW::e_d + N_Zgamma(M,alpha)*EW::e_d*v_d_Z + N_Z(M,alpha)*(v_d_Z*v_d_Z + a_d_Z*a_d_Z) ;
  }
  double lumi_DYn(double x1, double x2, double mu, double M, collider c) { // neutral (Z+gamma)
    using namespace PDF;
    //double alpha = EW::alpha_QED(M);
    double alpha = EW::alpha_em_Z;
    double pref_u = VsqAsq_u(M,alpha);
    double pref_d = VsqAsq_d(M,alpha);
    double res = 0;
    if (c==pp) {
      res = pref_u * ( u(x1,mu) * ubar(x2,mu) + ubar(x1,mu) * u(x2,mu) 
		       + 2. * chm(x1,mu) * chm(x2,mu) )
	+ pref_d * ( d(x1,mu) * dbar(x2,mu) + dbar(x1,mu) * d(x2,mu) 
		     + 2. * str(x1,mu) * str(x2,mu) 
		     + 2. * bot(x1,mu) * bot(x2,mu) );
    } else if (c==ppbar) {
      res = pref_u * ( u(x1,mu) * u(x2,mu) + ubar(x1,mu) * ubar(x2,mu) 
		       + 2. * chm(x1,mu) * chm(x2,mu) )
	+ pref_d * ( d(x1,mu) * d(x2,mu) + dbar(x1,mu) * dbar(x2,mu) 
		     + 2. * str(x1,mu) * str(x2,mu) 
		     + 2. * bot(x1,mu) * bot(x2,mu) );
    }
    return res;
  }
  double lumi_DYp(double x1, double x2, double mu, double M, collider c) { // positive (W+)
    using namespace PDF;
    double res = 0;
    if (c==pp) {
      res = u(x1,mu) * ( V_ud_sq * dbar(x2,mu) 
			 + V_us_sq * str(x2,mu) + V_ub_sq * bot(x2,mu) )
	+ chm(x1,mu) * ( V_cd_sq * dbar(x2,mu) 
			 + V_cs_sq * str(x2,mu) + V_cb_sq * bot(x2,mu) )
	+ u(x2,mu) * ( V_ud_sq * dbar(x1,mu) 
		       + V_us_sq * str(x1,mu) + V_ub_sq * bot(x1,mu) )
	+ chm(x2,mu) * ( V_cd_sq * dbar(x1,mu) 
			 + V_cs_sq * str(x1,mu) + V_cb_sq * bot(x1,mu) );
    } else if (c==ppbar) {
      res = u(x1,mu) * ( V_ud_sq * d(x2,mu) 
			 + V_us_sq * str(x2,mu) + V_ub_sq * bot(x2,mu) )
	+ chm(x1,mu) * ( V_cd_sq * d(x2,mu) 
			 + V_cs_sq * str(x2,mu) + V_cb_sq * bot(x2,mu) )
	+ ubar(x2,mu) * ( V_ud_sq * dbar(x1,mu) 
			  + V_us_sq * str(x1,mu) + V_ub_sq * bot(x1,mu) )
	+ chm(x2,mu) * ( V_cd_sq * dbar(x1,mu) 
			 + V_cs_sq * str(x1,mu) + V_cb_sq * bot(x1,mu) );
    }
    return res;
  }
  double lumi_DYm(double x1, double x2, double mu, double M, collider c) { // negative (W-)
    using namespace PDF;
    double res = 0;
    if (c==pp) {
      res = ubar(x1,mu) * ( V_ud_sq * d(x2,mu) 
			    + V_us_sq * str(x2,mu) + V_ub_sq * bot(x2,mu) )
	+ chm(x1,mu) * ( V_cd_sq * d(x2,mu) 
			 + V_cs_sq * str(x2,mu) + V_cb_sq * bot(x2,mu) )
	+ ubar(x2,mu) * ( V_ud_sq * d(x1,mu) 
			  + V_us_sq * str(x1,mu) + V_ub_sq * bot(x1,mu) )
	+ chm(x2,mu) * ( V_cd_sq * d(x1,mu) 
			 + V_cs_sq * str(x1,mu) + V_cb_sq * bot(x1,mu) );
    } else if (c==ppbar) {
      res = ubar(x1,mu) * ( V_ud_sq * dbar(x2,mu) 
			    + V_us_sq * str(x2,mu) + V_ub_sq * bot(x2,mu) )
	+ chm(x1,mu) * ( V_cd_sq * dbar(x2,mu) 
			 + V_cs_sq * str(x2,mu) + V_cb_sq * bot(x2,mu) )
	+ u(x2,mu) * ( V_ud_sq * d(x1,mu) 
		       + V_us_sq * str(x1,mu) + V_ub_sq * bot(x1,mu) )
	+ chm(x2,mu) * ( V_cd_sq * d(x1,mu) 
			 + V_cs_sq * str(x1,mu) + V_cb_sq * bot(x1,mu) );
    }
    return res;
  }
  //
  double lumi_DY(double x1, double x2, double mu, double M, process p, collider c) {
    double res;
    if(p == DYneutral) res = lumi_DYn(x1,x2,mu,M,c);
    else if(p == DYpositive) res = lumi_DYp(x1,x2,mu,M,c);
    else if(p == DYnegative) res = lumi_DYm(x1,x2,mu,M,c);
    else abort();
    return res;
  }




};


