#include "special_functions.hh"
#include <gsl/gsl_sf.h>
#include <sstream>
#include <iomanip>
#include <limits>



/*
double Round(double x, int n_decimals) {
  //x *= pow(10., n_decimals);
  //return pow(10., -n_decimals) * floor(x+0.5);
  stringstream ss;
  ss << setprecision(n_decimals) << x; // output the rounded number on the stream
  ss >> x; // put the rounded value back to the number 
  return x;
}
complex Round(complex x, int n_decimals) {
  return complex( Round( real(x), n_decimals ),
		  Round( imag(x), n_decimals ) );
}
*/

double ABS(double a){ return (a >0 ? a: -a); }
int    ABS(int a)   { return (a >0 ? a: -a); }

double  MIN(double a, double b) { return (a < b ? a : b); }
double  MAX(double a, double b) { return (a > b ? a : b); }
double SIGN(double a, double b) { return (b < 0 ? -ABS(a): ABS(a)); }

int  MIN(int a, int b) { return (a < b ? a : b); }
int  MAX(int a, int b) { return (a > b ? a : b); }
int SIGN(int a, int b) { return (b < 0 ? -ABS(a): ABS(a)); }


int INT(double a) { return (int) a; }
bool isINT(double a) {
  return (a-INT(a) == 0);
}







complex logGamma(complex z){
  gsl_sf_result r, arg;
  gsl_sf_lngamma_complex_e(real(z),imag(z),&r,&arg);  // causes problems in (x,0), x<0
  return r.val + I*arg.val;
}
complex Gamma(complex z){
  gsl_sf_result r, arg;
  gsl_sf_lngamma_complex_e(real(z),imag(z),&r,&arg);  // causes problems in (x,0), x<0
  return polar(exp(r.val),arg.val);
}






const double EPS   = numeric_limits<double>::epsilon();
const double FPMIN = numeric_limits<double>::min()/EPS;
//
complex betacf(const complex a, const complex b, const double x) { 
  int m, m2; 
  complex aa, c, d, del, h, qab, qam, qap; 
  qab = a+b;
  qap = a+1.;
  qam = a-1.; 
  c = 1.;
  d = 1. - qab*x/qap; 
  if (ABS(d) < FPMIN) d=FPMIN; 
  d = 1.0/d; 
  h=d; 
  for (m=1; m<10000; m++) { 
    m2 = 2*m; 
    aa = m*(b-m)*x/(qam+m2)/(a+m2); 
    d = 1. + aa*d;
    if (ABS(d) < FPMIN) d=FPMIN; 
    c = 1. + aa/c; 
    if (ABS(c) < FPMIN) c=FPMIN; 
    d = 1./d; 
    h *= d*c; 
    aa = -(a+m)*(qab+m)*x/(a+m2)/(qap+m2); 
    d = 1. + aa*d;
    if (ABS(d) < FPMIN) d=FPMIN; 
    c = 1. + aa/c; 
    if (ABS(c) < FPMIN) c=FPMIN; 
    d = 1./d; 
    del = d*c; 
    h *= del; 
    if (ABS(del-1.0) <= EPS) break;
  } 
  if(m==9999) cerr << "max iteration reached in betacf" << endl;
  return h; 
} 

// int_0^x dz z^(a-1) (1-z)^(b-1)
complex IncompleteBeta(const complex a, const complex b, const double x) {
  complex bt; 
  //if (a <= 0.0 || b <= 0.0) throw("Bad a or b in routine betai"); 
  if (x < 0.0 || x > 1.0) throw("Bad x in routine betai"); 
  //if (x == 0.0 || x == 1.0) return x; 
  if (x == 0.0) return 0; 
  if (x == 1.0) return exp(logGamma(a) + logGamma(b) - logGamma(a+b) ); 
  //if (a > SWITCH && b > SWITCH) return betaiapprox(a,b,x); 
  bt = exp( a*log(x) + b*log(1.-x) ); 
  if (x < real((a+1.0)/(a+b+2.0)) ) return bt * betacf(a,b,x)/a; 
  else return exp(logGamma(a) + logGamma(b) - logGamma(a+b) ) - bt * betacf(b,a,1.-x)/b; 
}




//struct Gamma : Gauleg18 {

//static const int ASWITCH=100;
complex gln;

complex gser(const complex a, const double x) {
  complex sum,del,ap;
  gln=logGamma(a);
  ap=a;
  del = sum = 1.0/a;
  for (;;) {
    ++ap;
    del *= x/ap;
    sum += del;
    if (ABS(del) < ABS(sum)*EPS) {
      return sum*exp(-x+a*log(x)-gln);
    }
  }
}
complex gcf(const complex a, const double x) {
  int i;
  complex an,b,c,d,del,h;
  gln=logGamma(a);
  b=x+1.0-a;
  c=1.0/FPMIN;
  d=1.0/b;
  h=d;
  for (i=1;;i++) {
    an = -i*(i-a);
    b += 2.0;
    d=an*d+b;
    if (ABS(d) < FPMIN) d=FPMIN;
    c=b+an/c;
    if (ABS(c) < FPMIN) c=FPMIN;
    d=1.0/d;
    del=d*c;
    h *= del;
    if (ABS(del-1.0) <= EPS) break;
  }
  return exp(-x+a*log(x)-gln)*h;
}
/*
double gammpapprox(double a, double x, int psig) {
  int j;
  double xu,t,sum,ans;
  double a1 = a-1.0, lna1 = log(a1), sqrta1 = sqrt(a1);
  gln = gammln(a);
  if (x > a1) xu = MAX(a1 + 11.5*sqrta1, x + 6.0*sqrta1);
  else xu = MAX(0.,MIN(a1 - 7.5*sqrta1, x - 5.0*sqrta1));
  sum = 0;
  for (j=0;j<ngau;j++) {
    t = x + (xu-x)*y[j];
    sum += w[j]*exp(-(t-a1)+a1*(log(t)-lna1));
  }
  ans = sum*(xu-x)*exp(a1*(lna1-1.)-gln);
  return (psig?(ans>0.0? 1.0-ans:-ans):(ans>=0.0? ans:1.0+ans));
}
*/
complex TruncatedGamma(const complex a, const double x) {
  complex res;
  if (x < 0.0) {
    cerr << "bad args in TruncatedGamma" << endl;
    exit(-72);
  }
  if (real(a) <= 0.0) { // recursion
    return (TruncatedGamma(a+1,x)+exp(-x+a*log(x)))/a;
  }
  if (x == 0.0) res=0.;
  //else if ((int)a >= ASWITCH) return gammpapprox(a,x,1); 
  else if (x < real(a)+1.0) res = gser(a,x);
  else res = 1. - gcf(a,x);
  return res*Gamma(a);
}
complex IncompleteGamma(const complex a, const double x) {
  complex res;
  if (x < 0.0) {
    cerr << "bad args in IncompleteGamma" << endl;
    exit(-71);
  }
  if (real(a) <= 0.0) { // recursion
    return (IncompleteGamma(a+1,x)-exp(-x+a*log(x)))/a;
  }
  if (x == 0.0) res= 1.;
  //else if ((int)a >= ASWITCH) return gammpapprox(a,x,0);
  else if (x < real(a)+1.0) res = 1.0-gser(a,x);
  else res = gcf(a,x);
  return res*Gamma(a);
}


complex psi(complex Z){
  complex SUB = 0. ;
  complex ZZ = Z;
  if(ABS(imag(ZZ))<10.) { // if too close to the real axis...
  label1:
    if(real(ZZ)<10.) { // ...use recurrence relation to push real(z) large enough
      SUB = SUB - 1./ ZZ;
      ZZ = ZZ + 1.;
      goto label1;
    }
  }
  complex RZ = 1./ ZZ;
  complex DZ = RZ * RZ;
  // SUB + asympt expansion (Abramowitz, Stengun, 6.3.18)
  return SUB + log(ZZ) - 0.5 * RZ - DZ/5040. * ( 420.+ DZ * ( - 42. + DZ * (20. - 21. * DZ) ) );
}



